/* Plot the spectral index map of a given data set

   What needs to be passed:

   float **regflux - An ...


Basic usage. We assume imgnum is already se...

  float **alpha;
  alpha = calloc(imgnum, sizeof(float *));

  ...
  specindex(flux, alpha, numregions, map1, map2, freq1, freq2, 1);

   If you have any question please email jeremy.harwood@physics.org and I will try and answer them as soon as I can
*/

//#include "mapscale.h"
//#include "drawbeam.h"

int plotspecindex(float **specindex, int numregions, int *xdim, int *ydim, float *minsi, float *maxsi, int ***regionarray, int currentset, char *specmaptitle, int zoom, int border, int titles, int labels, int axismarks, int symbol, int smoothmaps, float **flux, int extwincontrol, int contours, int contourlevels, int contourcolour, int contourlinestyle, int firstcontour, int contourlogs, char *output, float ra, float dec, float cellsize, int scaletype, float bmaj, float bmin, float bpa, int beamposx, int beamposy, int plotbeam, float wedgemultiplier, int xshift, int yshift) {

#include "smoothmaps.h"

  int mapscale(int scaletype, int istart, int iend, int jstart, int jend, int border, float cellsize, int axismarks, int labels, float ra, float dec, char *maptitle, int xshift, int yshift);

  int drawbeam(float xpos, float ypos, float bmaj, float bmin, float bpa);

  printf("Creating a spectral index map...\n");


  // Declaring variables

  int i, j, istart, jstart, iend, jend;
  float zoomedx, zoomedy, adj_maxrange;
 
  int intenscale = 0;

  // Declaring PGPLOT prototypes

  int cpgopen(const char *device);
  void cpgclos(void);
  void cpgenv(float xmin, float xmax, float ymin, float ymax, int just, int axis);
  void cpgpt1(float xpt, float ypt, int symbol);
  void cpgsci(int ci);
  void cpgimag(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, float a1, float a2, const float *tr);
  void cpgctab(const float *l, const float *r, const float *g, const float *b, int nc, float contra, float bright);
  void cpgwedg(const char *side, float disp, float width, float fg, float bg, const char *label);
  void cpgscr(int ci, float cr, float cg, float cb);
  void cpgcont(const float *a, int idim, int jdim, int i1, int i2, int j1, int j2, const float *c, int nc, const float *tr);
  void cpgsls(int ls);

  if (extwincontrol != 1) { 
    cpgopen(output);
  }

  // Swap to background and forground colours to something sensible
  cpgscr(0, 1, 1, 1);
  cpgscr(1, 0, 0, 0);

  cpgsci(1);

  // Calculate the zoomed start and end points
  zoomedx =  xdim[currentset] / zoom;
  zoomedy = ydim[currentset] / zoom;

  istart = xshift + border + ( (xdim[currentset] - (int)zoomedx) / 2);
  jstart = yshift + border + ( (ydim[currentset]  - (int)zoomedy) / 2);

  iend = xshift + xdim[currentset] - ( ( (xdim[currentset] - (int)zoomedx) / 2) + border);
  jend = yshift + ydim[currentset] - ( ( (ydim[currentset] - (int)zoomedy) / 2) + border);


  if (titles == 0) { sprintf(specmaptitle," ");  }

  mapscale(scaletype, istart, iend, jstart, jend, border, cellsize, axismarks, labels, ra, dec, specmaptitle, xshift, yshift);

  if (smoothmaps == 1) {

    float *tmp_smootharray;
    float minrange = 1e23;
    float maxrange = -1e23;
    int xaxismax = xdim[currentset];
    int yaxismax = ydim[currentset];
  
    tmp_smootharray = (float *)calloc(xaxismax*yaxismax, sizeof(float));

    /* // This is to try and allow moving outside of the data box e.g. istart = -50
    if (istart-border < 0) {
      for (i=istart-border; i < 0; i++) {
	for (j=0; j < yaxismax; j++) {
	  tmp_smootharray[(abs(istart-border)+i)*xaxismax+j] = 1e24;
	}
      }
    }

    if (jstart-border < 0) {

      for (j=jstart-border; j < 0; j++) {
	for (i=0; i < xaxismax; i++) {
	  tmp_smootharray[i*xaxismax+(abs(istart-border)+i)] = 1e24;
	}
      }
    }
    */

    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {
	
	if (regionarray[currentset][i][j] < 0) { // Set void regions to a high value to create a white background
	  tmp_smootharray[i*xaxismax+j] = 1e24;
	}

	else {

	  tmp_smootharray[i*xaxismax+j] = specindex[currentset][regionarray[currentset][i][j]];

	  if (tmp_smootharray[i*xaxismax+j] < minrange) {
	    minrange = tmp_smootharray[i*xaxismax+j];
	  }

	  if (tmp_smootharray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_smootharray[i*xaxismax+j];
	  }
	}

      }
    }

    adj_maxrange = maxrange * wedgemultiplier;

    if (adj_maxrange < 0.1) { // this prevents a 0 range, especially when using one region
      printf("\n*** Warning: Range is too small, setting maxrange to a value of 0.1\n\n"); 
      adj_maxrange = 0.1;
    }

    cpgctab(rl,rr,rg,rb,11,contra,bright);
    cpgwedg("RI", 1.5, 5.0, minrange, adj_maxrange, "Spectral Index");
    cpgimag(tmp_smootharray,xaxismax,yaxismax,istart,iend,jstart,jend,minrange,adj_maxrange,trans);

    free(tmp_smootharray);
  }

  else {

    // Use the original mapping style

    // Loop through setting and plotting each region

   for (i=border; i<(xdim[currentset]-border); i++){
      for (j=border; j<(ydim[currentset]-border); j++){

	if (regionarray[currentset][i][j] == -1) {
	  intenscale = 0;   
	}

	else if (regionarray[currentset][i][j] == -2) {
	  intenscale = 0;
	}

	else if (regionarray[currentset][i][j] == 0) {
	  intenscale = 0;
	}

	else if (specindex[currentset][regionarray[currentset][i][j]] > 1e33) {
	  intenscale = 0;
	}

	else {

	  if (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - ((maxsi[currentset]-minsi[currentset])/15)) ) {
	    intenscale = 1;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - ((maxsi[currentset]-minsi[currentset])/15))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (2*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 2;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (2*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (3*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 3;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (3*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (4*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 4;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (4*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (5*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 5;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (5*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (6*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 6;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (6*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (7*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 7;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (7*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (8*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 8;
	  }
	  
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (8*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (9*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 9;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (9*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (10*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 10;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (10*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (11*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 11;
	  }

	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (11*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (12*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 12;
	  }
	  
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (12*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (13*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 13;
	  }
	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (13*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= (maxsi[currentset] - (14*((maxsi[currentset]-minsi[currentset])/15)))) ) {
	    intenscale = 14;
	  }	 


	  else if ( (specindex[currentset][regionarray[currentset][i][j]] < (maxsi[currentset] - (14*((maxsi[currentset]-minsi[currentset])/15)))) && (specindex[currentset][regionarray[currentset][i][j]] >= minsi[currentset]) ) {
	    intenscale = 15;
	  }
	  
	   else {
	    intenscale = 0;
	    }
	}
	cpgsci(intenscale);
	cpgpt1(j,i,symbol);
      }
   }

}



  if (contours == 1) {

    float *tmp_contourarray;
    float minrange = 1e32;
    float maxrange = -1e32;
    float currentcont;
    int xaxismax = xdim[currentset];
    int yaxismax = ydim[currentset];


    cpgsci(contourcolour);
    cpgsls(contourlinestyle);

    tmp_contourarray = (float *)calloc(xaxismax*yaxismax, sizeof(float));

    for (i=0; i<xaxismax; i++) {
      for (j=0; j<yaxismax; j++) {

       	if (regionarray[currentset][i][j] > 0) { // Stop non-included regions from creating a false minimum

	  tmp_contourarray[i*xaxismax+j] = flux[i][j];

	  if (tmp_contourarray[i*xaxismax+j] < minrange) {
	    minrange = tmp_contourarray[i*xaxismax+j];
	  }

	  if (tmp_contourarray[i*xaxismax+j] > maxrange) {
	    maxrange = tmp_contourarray[i*xaxismax+j];
	  }
	}

	else {
	  tmp_contourarray[i*xaxismax+j] = 0.0;
	}

      }
    }


    for (i=firstcontour; i<=contourlevels; i++) {

      if (contourlogs == 1) {
	currentcont = pow(10, log10(minrange) + (i*(log10(maxrange)-log10(minrange))/contourlevels) );
      }
      else {
	currentcont = minrange + i*(maxrange-minrange)/contourlevels;
      }

      cpgcont(tmp_contourarray, xaxismax, yaxismax, istart, iend, jstart, jend, &currentcont, -1, trans);
    }

    free(tmp_contourarray);
  }


  // Plot the beam if the option is set to on
  if (plotbeam == 1) {
    int beamposi = istart-border+(bmaj/cellsize)+beamposx;
    int beamposj = jstart-border+(bmaj/cellsize)+beamposy;

    // This must be after the main plotting to create an overlay
    drawbeam(beamposi, beamposj, bmaj/cellsize, bmin/cellsize, bpa);
  }


  if (extwincontrol != 1) {
    cpgclos();
  }

  return 0;

}
